/*
  Copyright (c) 2023 PIXILAB Technologies AB, Sweden (http://pixilab.se). All Rights Reserved.
*/

import { SerialPort } from "../system/Network";
import { Driver } from "../system_lib/Driver";
import { driver, max, min, property } from "../system_lib/Metadata";

@driver("SerialPort", { baudRate: 57600 })
export class EncoderExample extends Driver<SerialPort> {
    private _position = 0; // Stores the reported encoder position

    constructor(private connection: SerialPort) {
        super(connection);
        connection.autoConnect();
        connection.subscribe("textReceived", (emitter, message) => {
            // When we receive a text message, we try to convert the text
            // message to a number.
            try {
                this._position = parseFloat(message.text);
                // We need to explicitly tell Blocks that the position property
                // has changed.
                this.changed("position");
            }
            catch (parseError) {
                console.error("Could not parse received text!");
            }
        });
    }

    @property("Encoder position")
    @min(0)
    @max(1)
    get position() {
        return this._position;
    }

    set position(newValue: number) {
        this._position = newValue;

        // Send the new value (5 decimals + carriage return) to the Arduino.
        this.connection.sendText(newValue.toFixed(5), "\n");
    }
}
